/*
 * Copyright (C) 2021-2023 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(WEBXR)

#include "ArgumentCoders.h"
#include "Connection.h"
#include "MessageNames.h"
#include "XRDeviceIdentifier.h"
#include <WebCore/PlatformXR.h>
#include <wtf/Forward.h>
#include <wtf/RuntimeApplicationChecks.h>
#include <wtf/ThreadSafeRefCounted.h>


namespace Messages {
namespace PlatformXRSystemProxy {

static inline IPC::ReceiverName messageReceiverName()
{
#if ASSERT_ENABLED
    static std::once_flag onceFlag;
    std::call_once(
        onceFlag,
        [&] {
            ASSERT(isInWebProcess());
        }
    );
#endif
    return IPC::ReceiverName::PlatformXRSystemProxy;
}

class SessionDidEnd {
public:
    using Arguments = std::tuple<WebKit::XRDeviceIdentifier>;

    static IPC::MessageName name() { return IPC::MessageName::PlatformXRSystemProxy_SessionDidEnd; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;

    explicit SessionDidEnd(const WebKit::XRDeviceIdentifier& deviceIdentifier)
        : m_arguments(deviceIdentifier)
    {
        ASSERT(!isInAuxiliaryProcess());
    }

    auto&& arguments()
    {
        return WTFMove(m_arguments);
    }

private:
    std::tuple<const WebKit::XRDeviceIdentifier&> m_arguments;
};

class SessionDidUpdateVisibilityState {
public:
    using Arguments = std::tuple<WebKit::XRDeviceIdentifier, PlatformXR::VisibilityState>;

    static IPC::MessageName name() { return IPC::MessageName::PlatformXRSystemProxy_SessionDidUpdateVisibilityState; }
    static constexpr bool isSync = false;
    static constexpr bool canDispatchOutOfOrder = false;
    static constexpr bool replyCanDispatchOutOfOrder = false;
    static constexpr bool deferSendingIfSuspended = false;

    SessionDidUpdateVisibilityState(const WebKit::XRDeviceIdentifier& deviceIdentifier, const PlatformXR::VisibilityState& visibilityState)
        : m_arguments(deviceIdentifier, visibilityState)
    {
        ASSERT(!isInAuxiliaryProcess());
    }

    auto&& arguments()
    {
        return WTFMove(m_arguments);
    }

private:
    std::tuple<const WebKit::XRDeviceIdentifier&, const PlatformXR::VisibilityState&> m_arguments;
};

} // namespace PlatformXRSystemProxy
} // namespace Messages

#endif // ENABLE(WEBXR)
